package mixins

import (
	dms3ld "gitlab.dms3.io/ld/go-ld-prime"
)

// Link can be embedded in a struct to provide all the methods that
// have fixed output for any int-kinded nodes.
// (Mostly this includes all the methods which simply return ErrWrongKind.)
// Other methods will still need to be implemented to finish conforming to Node.
//
// To conserve memory and get a TypeName in errors without embedding,
// write methods on your type with a body that simply initializes this struct
// and immediately uses the relevant method;
// this is more verbose in source, but compiles to a tighter result:
// in memory, there's no embed; and in runtime, the calls will be inlined
// and thus have no cost in execution time.
type Link struct {
	TypeName string
}

func (Link) Kind() dms3ld.Kind {
	return dms3ld.Kind_Link
}
func (x Link) LookupByString(string) (dms3ld.Node, error) {
	return nil, dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "LookupByString", AppropriateKind: dms3ld.KindSet_JustMap, ActualKind: dms3ld.Kind_Link}
}
func (x Link) LookupByNode(key dms3ld.Node) (dms3ld.Node, error) {
	return nil, dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "LookupByNode", AppropriateKind: dms3ld.KindSet_JustMap, ActualKind: dms3ld.Kind_Link}
}
func (x Link) LookupByIndex(idx int64) (dms3ld.Node, error) {
	return nil, dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "LookupByIndex", AppropriateKind: dms3ld.KindSet_JustList, ActualKind: dms3ld.Kind_Link}
}
func (x Link) LookupBySegment(dms3ld.PathSegment) (dms3ld.Node, error) {
	return nil, dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "LookupBySegment", AppropriateKind: dms3ld.KindSet_Recursive, ActualKind: dms3ld.Kind_Link}
}
func (Link) MapIterator() dms3ld.MapIterator {
	return nil
}
func (Link) ListIterator() dms3ld.ListIterator {
	return nil
}
func (Link) Length() int64 {
	return -1
}
func (Link) IsAbsent() bool {
	return false
}
func (Link) IsNull() bool {
	return false
}
func (x Link) AsBool() (bool, error) {
	return false, dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "AsBool", AppropriateKind: dms3ld.KindSet_JustBool, ActualKind: dms3ld.Kind_Link}
}
func (x Link) AsInt() (int64, error) {
	return 0, dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "AsInt", AppropriateKind: dms3ld.KindSet_JustInt, ActualKind: dms3ld.Kind_Link}
}
func (x Link) AsFloat() (float64, error) {
	return 0, dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "AsFloat", AppropriateKind: dms3ld.KindSet_JustFloat, ActualKind: dms3ld.Kind_Link}
}
func (x Link) AsString() (string, error) {
	return "", dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "AsString", AppropriateKind: dms3ld.KindSet_JustString, ActualKind: dms3ld.Kind_Link}
}
func (x Link) AsBytes() ([]byte, error) {
	return nil, dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "AsBytes", AppropriateKind: dms3ld.KindSet_JustBytes, ActualKind: dms3ld.Kind_Link}
}

// LinkAssembler has similar purpose as Link, but for (you guessed it)
// the NodeAssembler interface rather than the Node interface.
type LinkAssembler struct {
	TypeName string
}

func (x LinkAssembler) BeginMap(sizeHint int64) (dms3ld.MapAssembler, error) {
	return nil, dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "BeginMap", AppropriateKind: dms3ld.KindSet_JustMap, ActualKind: dms3ld.Kind_Link}
}
func (x LinkAssembler) BeginList(sizeHint int64) (dms3ld.ListAssembler, error) {
	return nil, dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "BeginList", AppropriateKind: dms3ld.KindSet_JustList, ActualKind: dms3ld.Kind_Link}
}
func (x LinkAssembler) AssignNull() error {
	return dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "AssignNull", AppropriateKind: dms3ld.KindSet_JustNull, ActualKind: dms3ld.Kind_Link}
}
func (x LinkAssembler) AssignBool(bool) error {
	return dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "AssignBool", AppropriateKind: dms3ld.KindSet_JustBool, ActualKind: dms3ld.Kind_Link}
}
func (x LinkAssembler) AssignInt(int64) error {
	return dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "AssignInt", AppropriateKind: dms3ld.KindSet_JustInt, ActualKind: dms3ld.Kind_Link}
}
func (x LinkAssembler) AssignFloat(float64) error {
	return dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "AssignFloat", AppropriateKind: dms3ld.KindSet_JustFloat, ActualKind: dms3ld.Kind_Link}
}
func (x LinkAssembler) AssignString(string) error {
	return dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "AssignString", AppropriateKind: dms3ld.KindSet_JustString, ActualKind: dms3ld.Kind_Link}
}
func (x LinkAssembler) AssignBytes([]byte) error {
	return dms3ld.ErrWrongKind{TypeName: x.TypeName, MethodName: "AssignBytes", AppropriateKind: dms3ld.KindSet_JustBytes, ActualKind: dms3ld.Kind_Link}
}
